@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -44,16 +44,24 @@
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
 #include <ofono/radio-settings.h>
+#include <ofono/voicecall.h>
 #include <ofono/log.h>
 
 #include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
 
 static const char *none_prefix[] = { NULL };
+static const char *opmn_prefix[] = { "_OPMN:", NULL };
+static const char *obls_prefix[] = { "_OBLS:", NULL };
+static const char *opcm_prefix[] = { "_OPCMENABLE:", NULL };
 
 struct hso_data {
 	GAtChat *app;
 	GAtChat *control;
+	guint sim_poll_source;
+	guint sim_poll_count;
+	ofono_bool_t have_sim;
+	ofono_bool_t have_voice;
 };
 
 static int hso_probe(struct ofono_modem *modem)
@@ -63,7 +71,7 @@ static int hso_probe(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	data = g_try_new0(struct hso_data, 1);
-	if (!data)
+	if (data == NULL)
 		return -ENOMEM;
 
 	ofono_modem_set_data(modem, data);
@@ -80,27 +88,145 @@ static void hso_remove(struct ofono_modem *modem)
 	ofono_modem_set_data(modem, NULL);
 
 	g_at_chat_unref(data->control);
+
+	if (data->sim_poll_source > 0)
+		g_source_remove(data->sim_poll_source);
+
 	g_free(data);
 }
 
 static void hso_debug(const char *str, void *user_data)
 {
 	const char *prefix = user_data;
+
 	ofono_info("%s%s", prefix, str);
 }
 
-static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
+static void opcm_query(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct hso_data *data = ofono_modem_get_data(modem);
+	GAtResultIter iter;
 
 	DBG("");
 
-	ofono_modem_set_powered(modem, ok);
+	if (!ok)
+		return;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "_OPCMENABLE:"))
+		return;
+
+	g_at_chat_send(data->app, "AT_OPCMENABLE=1", none_prefix,
+						NULL, NULL, NULL);
+}
+
+static void opcm_support(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct hso_data *data = ofono_modem_get_data(modem);
+	GAtResultIter iter;
+
+	DBG("");
 
 	if (!ok)
 		return;
 
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "_OPCMENABLE:"))
+		return;
+
+	data->have_voice = TRUE;
+
+	g_at_chat_send(data->app, "AT_OPCMENABLE?", opcm_prefix,
+					opcm_query, modem, NULL);
+}
+
+static gboolean init_sim_check(gpointer user_data);
+
+static void sim_status(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct hso_data *data = ofono_modem_get_data(modem);
+	GAtResultIter iter;
+	int sim, pb, sms;
+
+	DBG("");
+
+	if (data->sim_poll_source > 0) {
+		g_source_remove(data->sim_poll_source);
+		data->sim_poll_source = 0;
+	}
+
+	if (!ok) {
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "_OBLS:")) {
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	if (!g_at_result_iter_next_number(&iter, &sim)) {
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	if (!g_at_result_iter_next_number(&iter, &pb)) {
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	if (!g_at_result_iter_next_number(&iter, &sms)) {
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	DBG("status sim %d pb %d sms %d", sim, pb, sms);
+
+	switch (sim) {
+	case 0:		/* not ready */
+		data->have_sim = FALSE;
+
+		if (data->sim_poll_count++ < 5) {
+			data->sim_poll_source = g_timeout_add_seconds(1,
+							init_sim_check, modem);
+			return;
+		}
+		break;
+	case 1:		/* SIM card ready */
+		data->have_sim = TRUE;
+		break;
+	case 2:		/* no SIM card */
+		data->have_sim = FALSE;
+		break;
+	default:
+		data->have_sim = FALSE;
+		break;
+	}
+
+	data->sim_poll_count = 0;
+
+	ofono_modem_set_powered(modem, data->have_sim);
+
+	if (data->have_sim == FALSE)
+		return;
+
+	/*
+	 * Ensure that the modem is using GSM character set and not IRA,
+	 * otherwise weirdness with umlauts and other non-ASCII characters
+	 * can result
+	 */
+	g_at_chat_send(data->control, "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
+	g_at_chat_send(data->app, "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
+
 	/*
 	 * Option has the concept of Speech Service versus
 	 * Data Service. Problem is that in Data Service mode
@@ -114,6 +240,61 @@ static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 	 */
 	g_at_chat_send(data->app, "AT_ODO?", none_prefix, NULL, NULL, NULL);
 	g_at_chat_send(data->app, "AT_ODO=0", none_prefix, NULL, NULL, NULL);
+
+	g_at_chat_send(data->app, "AT_OPCMENABLE=?", opcm_prefix,
+					opcm_support, modem, NULL);
+}
+
+static gboolean init_sim_check(gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct hso_data *data = ofono_modem_get_data(modem);
+
+	data->sim_poll_source = 0;
+
+	g_at_chat_send(data->control, "AT_OBLS", obls_prefix,
+					sim_status, modem, NULL);
+
+	return FALSE;
+}
+
+static void check_model(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	GAtResultIter iter;
+	char const *model;
+
+	DBG("");
+
+	if (!ok)
+		goto done;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "_OPMN:"))
+		goto done;
+
+	if (g_at_result_iter_next_unquoted_string(&iter, &model))
+		ofono_info("Model is %s", model);
+
+done:
+	init_sim_check(modem);
+}
+
+static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct hso_data *data = ofono_modem_get_data(modem);
+
+	DBG("");
+
+	if (!ok) {
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	g_at_chat_send(data->control, "AT_OPMN", opmn_prefix,
+					check_model, modem, NULL);
 }
 
 static GAtChat *create_port(const char *device)
@@ -123,7 +304,7 @@ static GAtChat *create_port(const char *device)
 	GAtChat *chat;
 
 	channel = g_at_tty_open(device, NULL);
-	if (!channel)
+	if (channel == NULL)
 		return NULL;
 
 	syntax = g_at_syntax_new_gsm_permissive();
@@ -131,7 +312,7 @@ static GAtChat *create_port(const char *device)
 	g_at_syntax_unref(syntax);
 	g_io_channel_unref(channel);
 
-	if (!chat)
+	if (chat == NULL)
 		return NULL;
 
 	return chat;
@@ -148,7 +329,7 @@ static int hso_enable(struct ofono_modem *modem)
 	control = ofono_modem_get_string(modem, "ControlPort");
 	app = ofono_modem_get_string(modem, "ApplicationPort");
 
-	if (!app || !control)
+	if (app == NULL || control == NULL)
 		return -EINVAL;
 
 	data->control = create_port(control);
@@ -157,7 +338,7 @@ static int hso_enable(struct ofono_modem *modem)
 		return -EIO;
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->control, hso_debug, "Control:");
+		g_at_chat_set_debug(data->control, hso_debug, "Control: ");
 
 	data->app = create_port(app);
 
@@ -169,10 +350,10 @@ static int hso_enable(struct ofono_modem *modem)
 	}
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->app, hso_debug, "App:");
+		g_at_chat_set_debug(data->app, hso_debug, "App: ");
 
-	g_at_chat_send(data->control, "ATE0", none_prefix, NULL, NULL, NULL);
-	g_at_chat_send(data->app, "ATE0", none_prefix, NULL, NULL, NULL);
+	g_at_chat_send(data->control, "ATE0 +CMEE=1", NULL, NULL, NULL, NULL);
+	g_at_chat_send(data->app, "ATE0 +CMEE=1", NULL, NULL, NULL, NULL);
 
 	g_at_chat_send(data->control, "AT+CFUN=4", none_prefix,
 					cfun_enable, modem, NULL);
@@ -200,7 +381,7 @@ static int hso_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	if (!data->control)
+	if (data->control == NULL)
 		return 0;
 
 	g_at_chat_cancel_all(data->control);
@@ -236,13 +417,9 @@ static void hso_set_online(struct ofono_modem *modem, ofono_bool_t online,
 
 	DBG("modem %p %s", modem, online ? "online" : "offline");
 
-	if (!cbd)
-		goto error;
-
 	if (g_at_chat_send(chat, command, NULL, set_online_cb, cbd, g_free))
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
@@ -257,9 +434,9 @@ static void hso_pre_sim(struct ofono_modem *modem)
 
 	ofono_devinfo_create(modem, 0, "atmodem", data->control);
 	sim = ofono_sim_create(modem, OFONO_VENDOR_OPTION_HSO,
-				"atmodem", data->control);
+					"atmodem", data->control);
 
-	if (sim)
+	if (sim && data->have_sim == TRUE)
 		ofono_sim_inserted_notify(sim, TRUE);
 }
 
@@ -269,7 +446,15 @@ static void hso_post_sim(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	if (data->have_voice == TRUE) {
+		ofono_voicecall_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+						"atmodem", data->app);
+	}
+
 	ofono_phonebook_create(modem, 0, "atmodem", data->app);
+	ofono_radio_settings_create(modem, 0, "hsomodem", data->app);
+
+	ofono_sms_create(modem, OFONO_VENDOR_OPTION_HSO, "atmodem", data->app);
 }
 
 static void hso_post_online(struct ofono_modem *modem)
@@ -281,15 +466,12 @@ static void hso_post_online(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_netreg_create(modem, OFONO_VENDOR_OPTION_HSO,
-				"atmodem", data->app);
+					"atmodem", data->app);
 
-	ofono_radio_settings_create(modem, 0, "hsomodem", data->app);
-
-	ofono_sms_create(modem, OFONO_VENDOR_OPTION_HSO, "atmodem", data->app);
 	ofono_cbs_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
-				"atmodem", data->app);
+					"atmodem", data->app);
 	ofono_ussd_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
-				"atmodem", data->app);
+					"atmodem", data->app);
 
 	gprs = ofono_gprs_create(modem, 0, "atmodem", data->app);
 	gc = ofono_gprs_context_create(modem, 0, "hsomodem", data->control);
@@ -304,7 +486,7 @@ static struct ofono_modem_driver hso_driver = {
 	.remove		= hso_remove,
 	.enable		= hso_enable,
 	.disable	= hso_disable,
-	.set_online     = hso_set_online,
+	.set_online	= hso_set_online,
 	.pre_sim	= hso_pre_sim,
 	.post_sim	= hso_post_sim,
 	.post_online	= hso_post_online,
